@@ -1,3 +1,3 @@
-from performer_pytorch.performer_pytorch import PerformerLM, Performer, FastAttention, SelfAttention
+from performer_pytorch.performer_pytorch import PerformerLM, Performer, FastAttention, SelfAttention, CrossAttention, ProjectionUpdater
 from performer_pytorch.autoregressive_wrapper import AutoregressiveWrapper
 from performer_pytorch.performer_enc_dec import PerformerEncDec
