@@ -12,6 +12,10 @@ from local_attention import LocalAttention
 from axial_positional_embedding import AxialPositionalEmbedding
 from performer_pytorch.reversible import ReversibleSequence, SequentialSequence
 
+from distutils.version import LooseVersion
+
+TORCH_GE_1_8_0 = LooseVersion(torch.__version__) >= LooseVersion('1.8.0')
+
 try:
     from apex import amp
     APEX_AVAILABLE = True
@@ -50,6 +54,34 @@ class Always(nn.Module):
     def forward(self, *args, **kwargs):
         return self.val
 
+# token shifting helper and classes
+
+def shift(t, amount, mask = None):
+    if amount == 0:
+        return t
+
+    if exists(mask):
+        t = t.masked_fill(~mask[..., None], 0.)
+
+    return F.pad(t, (0, 0, amount, -amount), value = 0.)
+
+class PreShiftTokens(nn.Module):
+    def __init__(self, shifts, fn):
+        super().__init__()
+        self.fn = fn
+        self.shifts = tuple(shifts)
+
+    def forward(self, x, **kwargs):
+        mask = kwargs.get('mask', None)
+        shifts = self.shifts
+        segments = len(shifts)
+        feats_per_shift = x.shape[-1] // segments
+        splitted = x.split(feats_per_shift, dim = -1)
+        segments_to_shift, rest = splitted[:segments], splitted[segments:]
+        segments_to_shift = list(map(lambda args: shift(*args, mask = mask), zip(segments_to_shift, shifts)))
+        x = torch.cat((*segments_to_shift, *rest), dim = -1)
+        return self.fn(x, **kwargs)
+
 # kernel functions
 
 # transcribed from jax to pytorch from
@@ -75,10 +107,10 @@ def softmax_kernel(data, *, projection_matrix, is_query, normalize_data=True, ep
     if is_query:
         data_dash = ratio * (
             torch.exp(data_dash - diag_data -
-                    torch.max(data_dash, dim=-1, keepdim=True).values) + eps)
+                    torch.amax(data_dash, dim=-1, keepdim=True)) + eps)
     else:
         data_dash = ratio * (
-            torch.exp(data_dash - diag_data - torch.max(data_dash)) + eps)
+            torch.exp(data_dash - diag_data - torch.amax(data_dash, dim=(-1, -2), keepdim=True)) + eps)
 
     return data_dash.type_as(data)
 
@@ -100,7 +132,10 @@ def generalized_kernel(data, *, projection_matrix, kernel_fn = nn.ReLU(), kernel
 
 def orthogonal_matrix_chunk(cols, device = None):
     unstructured_block = torch.randn((cols, cols), device = device)
-    q, r = torch.qr(unstructured_block.cpu(), some = True)
+    if TORCH_GE_1_8_0:
+        q, r = torch.linalg.qr(unstructured_block.cpu(), mode = 'reduced')
+    else:
+        q, r = torch.qr(unstructured_block.cpu(), some = True)
     q, r = map(lambda t: t.to(device), (q, r))
     return q.t()
 
@@ -141,7 +176,7 @@ def linear_attention(q, k, v):
 
 # efficient causal linear attention, created by EPFL
 # TODO: rewrite EPFL's CUDA kernel to do mixed precision and remove half to float conversion and back
-def causal_linear_attention(q, k, v):
+def causal_linear_attention(q, k, v, eps = 1e-6):
     from fast_transformers.causal_product import CausalDotProduct
     autocast_enabled = torch.is_autocast_enabled()
     is_half = isinstance(q, torch.cuda.HalfTensor)
@@ -150,7 +185,7 @@ def causal_linear_attention(q, k, v):
 
     causal_dot_product_fn = amp.float_function(CausalDotProduct.apply) if is_half else CausalDotProduct.apply
 
-    k_cumsum = k.cumsum(dim=-2)
+    k_cumsum = k.cumsum(dim=-2) + eps
     D_inv = 1. / torch.einsum('...nd,...nd->...n', q, k_cumsum.type_as(q))
 
     with cuda_context():
@@ -164,7 +199,7 @@ def causal_linear_attention(q, k, v):
 
 # inefficient causal linear attention, without cuda code, for reader's reference
 # not being used
-def causal_linear_attention_noncuda(q, k, v, chunk_size = 128):
+def causal_linear_attention_noncuda(q, k, v, chunk_size = 128, eps = 1e-6):
     last_k_cumsum = 0
     last_context_cumsum = 0
     outs = []
@@ -172,7 +207,7 @@ def causal_linear_attention_noncuda(q, k, v, chunk_size = 128):
     for q, k, v in zip(*map(lambda t: t.chunk(chunk_size, dim = -2), (q, k, v))):
         k_cumsum = last_k_cumsum + k.cumsum(dim=-2)
 
-        D_inv = 1. / torch.einsum('...nd,...nd->...n', q, k_cumsum.type_as(q))
+        D_inv = 1. / torch.einsum('...nd,...nd->...n', q, k_cumsum.type_as(q) + eps)
         context = torch.einsum('...nd,...ne->...nde', k, v)
         context_cumsum = last_context_cumsum + context.cumsum(dim=-3)
         out = torch.einsum('...nde,...nd,...n->...ne', context_cumsum, q, D_inv)
@@ -238,6 +273,39 @@ class FastAttention(nn.Module):
         out = attn_fn(q, k, v)
         return out
 
+# a module for keeping track of when to update the projections
+
+class ProjectionUpdater(nn.Module):
+    def __init__(self, instance, feature_redraw_interval):
+        super().__init__()
+        self.instance = instance
+        self.feature_redraw_interval = feature_redraw_interval
+        self.register_buffer('calls_since_last_redraw', torch.tensor(0))
+
+    def fix_projections_(self):
+        self.feature_redraw_interval = None
+
+    def redraw_projections(self):
+        model = self.instance
+
+        if not self.training:
+            return
+
+        if exists(self.feature_redraw_interval) and self.calls_since_last_redraw >= self.feature_redraw_interval:
+            device = get_module_device(model)
+
+            fast_attentions = find_modules(model, FastAttention)
+            for fast_attention in fast_attentions:
+                fast_attention.redraw_projection_matrix(device)
+
+            self.calls_since_last_redraw.zero_()
+            return
+
+        self.calls_since_last_redraw += 1
+
+    def forward(self, x):
+        raise NotImplemented
+
 # classes
 
 class ReZero(nn.Module):
@@ -305,7 +373,7 @@ class FeedForward(nn.Module):
         x = self.w2(x)
         return x
 
-class SelfAttention(nn.Module):
+class Attention(nn.Module):
     def __init__(
         self,
         dim,
@@ -320,7 +388,8 @@ class SelfAttention(nn.Module):
         kernel_fn = nn.ReLU(),
         dropout = 0.,
         no_projection = False,
-        qkv_bias = False
+        qkv_bias = False,
+        attn_out_bias = True
     ):
         super().__init__()
         assert dim % heads == 0, 'dimension must be divisible by number of heads'
@@ -335,7 +404,7 @@ class SelfAttention(nn.Module):
         self.to_q = nn.Linear(dim, inner_dim, bias = qkv_bias)
         self.to_k = nn.Linear(dim, inner_dim, bias = qkv_bias)
         self.to_v = nn.Linear(dim, inner_dim, bias = qkv_bias)
-        self.to_out = nn.Linear(inner_dim, dim)
+        self.to_out = nn.Linear(inner_dim, dim, bias = attn_out_bias)
         self.dropout = nn.Dropout(dropout)
 
     def forward(self, x, pos_emb = None, context = None, mask = None, context_mask = None, **kwargs):
@@ -351,9 +420,6 @@ class SelfAttention(nn.Module):
         q, k, v = map(lambda t: rearrange(t, 'b n (h d) -> b h n d', h = h), (q, k, v))
         (q, lq), (k, lk), (v, lv) = map(lambda t: (t[:, :gh], t[:, gh:]), (q, k, v))
 
-        if exists(pos_emb) and not cross_attend:
-            q, k, = apply_rotory_pos_emb(q, k, pos_emb)
-
         attn_outs = []
 
         if not empty(q):
@@ -361,6 +427,9 @@ class SelfAttention(nn.Module):
                 global_mask = context_mask[:, None, :, None]
                 v.masked_fill_(~global_mask, 0.)
 
+            if exists(pos_emb) and not cross_attend:
+                q, k = apply_rotary_pos_emb(q, k, pos_emb)
+
             out = self.fast_attention(q, k, v)
             attn_outs.append(out)
 
@@ -374,6 +443,16 @@ class SelfAttention(nn.Module):
         out =  self.to_out(out)
         return self.dropout(out)
 
+class SelfAttention(Attention):
+    def forward(self, *args, context = None, **kwargs):
+        assert not exists(context), 'self attention should not receive context'
+        return super().forward(*args, **kwargs)
+
+class CrossAttention(Attention):
+    def forward(self, *args, context = None, **kwargs):
+        assert exists(context), 'cross attention should receive context'
+        return super().forward(*args, context = context, **kwargs)
+
 # positional embeddings
 
 class AbsolutePositionalEmbedding(nn.Module):
@@ -393,7 +472,7 @@ def rotate_every_two(x):
     x = torch.stack((-x2, x1), dim = -1)
     return rearrange(x, '... d j -> ... (d j)')
 
-def apply_rotory_pos_emb(q, k, sinu_pos):
+def apply_rotary_pos_emb(q, k, sinu_pos):
     sinu_pos = rearrange(sinu_pos, '() n (j d) -> n j d', j = 2)
     sin, cos = sinu_pos.unbind(dim = -2)
     sin, cos = map(lambda t: repeat(t, 'b n -> b (n j)', j = 2), (sin, cos))
@@ -441,7 +520,9 @@ class Performer(nn.Module):
         cross_attend = False,
         no_projection = False,
         auto_check_redraw = True,
-        qkv_bias = True
+        qkv_bias = True,
+        attn_out_bias = True,
+        shift_tokens = False
     ):
         super().__init__()
         layers = nn.ModuleList([])
@@ -458,16 +539,22 @@ class Performer(nn.Module):
             wrapper_fn = partial(PreLayerNorm, dim)
 
         for _, local_heads in zip(range(depth), local_attn_heads):
-            layers.append(nn.ModuleList([
-                wrapper_fn(SelfAttention(dim, causal = causal, heads = heads, dim_head = dim_head, local_heads = local_heads, local_window_size = local_window_size, nb_features = nb_features, generalized_attention = generalized_attention, kernel_fn = kernel_fn, dropout = attn_dropout, no_projection = no_projection, qkv_bias = qkv_bias)),
-                wrapper_fn(Chunk(ff_chunks, FeedForward(dim, mult = ff_mult, dropout = ff_dropout, glu = ff_glu), along_dim = 1))
-            ]))
+
+            attn = SelfAttention(dim, causal = causal, heads = heads, dim_head = dim_head, local_heads = local_heads, local_window_size = local_window_size, nb_features = nb_features, generalized_attention = generalized_attention, kernel_fn = kernel_fn, dropout = attn_dropout, no_projection = no_projection, qkv_bias = qkv_bias, attn_out_bias = attn_out_bias)
+            ff = Chunk(ff_chunks, FeedForward(dim, mult = ff_mult, dropout = ff_dropout, glu = ff_glu), along_dim = 1)
+
+            if shift_tokens:
+                shift = (0, 1) if causal else (-1, 0, 1)
+                attn, ff = map(lambda t: PreShiftTokens(shift, t), (attn, ff))
+
+            attn, ff = map(wrapper_fn, (attn, ff))
+            layers.append(nn.ModuleList([attn, ff]))
 
             if not cross_attend:
                 continue
 
             layers.append(nn.ModuleList([
-                wrapper_fn(SelfAttention(dim, heads = heads, dim_head = dim_head, nb_features = nb_features, generalized_attention = generalized_attention, kernel_fn = kernel_fn, dropout = attn_dropout, no_projection = no_projection)),
+                wrapper_fn(CrossAttention(dim, heads = heads, dim_head = dim_head, nb_features = nb_features, generalized_attention = generalized_attention, kernel_fn = kernel_fn, dropout = attn_dropout, no_projection = no_projection, qkv_bias = qkv_bias, attn_out_bias = attn_out_bias)),
                 wrapper_fn(Chunk(ff_chunks, FeedForward(dim, mult = ff_mult, dropout = ff_dropout, glu = ff_glu), along_dim = 1))
             ]))
 
@@ -481,31 +568,14 @@ class Performer(nn.Module):
 
         # keeping track of when to redraw projections for all attention layers
         self.auto_check_redraw = auto_check_redraw
-        self.feature_redraw_interval = feature_redraw_interval
-        self.register_buffer('calls_since_last_redraw', torch.tensor(0))
+        self.proj_updater = ProjectionUpdater(self.net, feature_redraw_interval)
 
     def fix_projection_matrices_(self):
-        self.feature_redraw_interval = None
-
-    def check_redraw_projections(self):
-        if not self.training:
-            return
-
-        if exists(self.feature_redraw_interval) and self.calls_since_last_redraw >= self.feature_redraw_interval:
-            device = get_module_device(self)
-
-            fast_attentions = find_modules(self, FastAttention)
-            for fast_attention in fast_attentions:
-                fast_attention.redraw_projection_matrix(device)
-
-            self.calls_since_last_redraw.zero_()
-            return
-
-        self.calls_since_last_redraw += 1
+        self.proj_updater.feature_redraw_interval = None
 
     def forward(self, x, **kwargs):
         if self.auto_check_redraw:
-            self.check_redraw_projections()
+            self.proj_updater.redraw_projections()
         return self.net(x, **kwargs)
 
 class PerformerLM(nn.Module):
@@ -541,7 +611,9 @@ class PerformerLM(nn.Module):
         axial_position_emb = False,
         axial_position_shape = None,
         auto_check_redraw = True,
-        qkv_bias = False
+        qkv_bias = False,
+        attn_out_bias = False,
+        shift_tokens = False
     ):
         super().__init__()
         local_attn_heads = cast_tuple(local_attn_heads)
@@ -550,7 +622,7 @@ class PerformerLM(nn.Module):
         self.token_emb = nn.Embedding(num_tokens, dim)
 
         if rotary_position_emb:
-            self.pos_emb = Always(0)
+            self.pos_emb = FixedPositionalEmbedding(dim, max_seq_len)
             self.layer_pos_emb = FixedPositionalEmbedding(dim_head, max_seq_len)
         elif axial_position_emb:
             axial_position_shape = default(axial_position_shape, (math.ceil(max_seq_len / 64), 64))
@@ -562,7 +634,7 @@ class PerformerLM(nn.Module):
 
         self.dropout = nn.Dropout(emb_dropout)
 
-        self.performer = Performer(dim, depth, heads, dim_head, local_attn_heads, local_window_size, causal, ff_mult, nb_features, feature_redraw_interval, reversible, ff_chunks, generalized_attention, kernel_fn, use_scalenorm, use_rezero, ff_glu, ff_dropout, attn_dropout, cross_attend, no_projection, auto_check_redraw, qkv_bias)
+        self.performer = Performer(dim, depth, heads, dim_head, local_attn_heads, local_window_size, causal, ff_mult, nb_features, feature_redraw_interval, reversible, ff_chunks, generalized_attention, kernel_fn, use_scalenorm, use_rezero, ff_glu, ff_dropout, attn_dropout, cross_attend, no_projection, auto_check_redraw, qkv_bias, attn_out_bias, shift_tokens)
         self.norm = nn.LayerNorm(dim)
         self.to_out = nn.Linear(dim, num_tokens) if not tie_embed else None
 
